<?php
/**
 * Class to manage Gregorian calendar
 *
 * @category   Jaws_Date
 * @package    Core
 * @author     Amir Mohammad Saied <amir@php.net>
 * @copyright  2006-2012 Jaws Development Group
 * @license    http://www.gnu.org/copyleft/lesser.html
 */
class Jaws_Date_Gregorian extends Jaws_Date
{
    /**
     *
     * @param   int $year   Gregorian year
     * @param   int $month  Gregorian month
     * @param   int $day    Gregorian day
     * @access  public
     * @return  array   Converted time
     */
    function ToBaseDate($year, $month = 1, $day = 1, $hour = 0, $minute = 0, $second = 0, $format = '')
    {
        if (is_array(func_get_arg(0))) {
            @list($year, $month, $day, $hour, $minute, $second) = func_get_arg(0);
            $format = func_get_arg(1);
        }

        $dt = mktime($hour, $minute, $second, $month, $day, $year);
        return !empty($format)? date($format, $dt) :
                                array('timestamp' => $dt,
                                      'year'      => date("Y", $dt),
                                      'month'     => date("m", $dt),
                                      'day'       => date("d", $dt),
                                      'hour'      => date("H", $dt),
                                      'minute'    => date("i", $dt),
                                      'second'    => date("s", $dt),
                                      'monthDays' => date("t", $dt),
                                      'yearDay'   => date("z", $dt)
                                    );
    }

    /**
     * Get date information
     *
     * @param   int $year  Jalali year
     * @param   int $month Jalali month
     * @param   int $day   Jalali day
     * @access  public
     * @return  array   Date time information
     */
    function GetDateInfo($year, $month = 1, $day = 1, $hour = 0, $minute = 0, $second = 0)
    {
        if (is_array(func_get_arg(0))) {
            @list($year, $month, $day, $hour, $minute, $second) = func_get_arg(0);
            $date = mktime($hour, $minute, $second, $month, $day, $year);
        } elseif (func_num_args() > 1) {
            $date = mktime($hour, $minute, $second, $month, $day, $year);
        } else { // unix timestamp
            $date = $year;
        }

        $date = getdate($date);
        return array(
                'seconds' => str_pad($date['seconds'], 2, '0', STR_PAD_LEFT),
                'minutes' => str_pad($date['minutes'], 2, '0', STR_PAD_LEFT),
                'hours'   => str_pad($date['hours'],   2, '0', STR_PAD_LEFT),
                'mday'    => str_pad($date['mday'],    2, '0', STR_PAD_LEFT),
                'wday'    => $date['wday'],
                'mon'     => str_pad($date['mon'],  2, '0', STR_PAD_LEFT),
                'year'    => str_pad($date['year'], 4, '0', STR_PAD_LEFT),
                'yday'    => $date['yday'],
                'weekday' => $this->DayString($date['wday']),
                'month'   => $this->MonthString($date['mon']),
            );
    }

    /**
     * Format the input date.
     *
     * @param  string  $date   Date string
     * @param  string  $format Format to use
     * @return The original date with a new format
     */
    function Format($date, $format = null)
    {
        if (empty($date)) {
            return '';
        }

        $date = $GLOBALS['app']->UTC2UserTime($date);

        if (empty($format)) {
            $format = $GLOBALS['app']->Registry->Get('/config/date_format');
        }

        if ($format == 'since') {
            return $this->SinceFormat($date);
        } else {
            $i = 0; 
            $return = '';
            while ($i < strlen($format)) {
                switch($format[$i]) {
                case 'A':
                    if (substr($format, $i, 3) == 'AGO') {
                        $return .= $this->SinceFormat($date);
                        $i = $i + 2;
                    }
                    break;
                case 'D':
                    if (substr($format, $i, 2) == 'DN') {
                        $return .= $this->DayString(date('w', $date));
                        $i++;
                    } else {
                        $return .= $this->DayShortString(date('w', $date));
                    }
                    break;
                case 'M':
                    if (substr($format, $i, 2) == 'MN') {
                        $return .= $this->MonthString(date('m', $date));
                        $i++;
                    } else {
                        $return .= $this->MonthShortString(date('m', $date));
                    }
                    break;
                case '\\':
                    // Do nothing 
                    break;
                default:
                    if (substr($format, $i - 1, 1) == '\\') {
                        $return .= $format[$i];
                    } else {
                        $return .= date($format[$i], $date);
                    }
                    break;
                }
                $i++;
            }

            return $return;
                 
        }
    }
}
